using System;
using Microsoft.SPOT;
using Gadgeteer.Modules.GHIElectronics;
using GT = Gadgeteer;
using GTM = Gadgeteer.Modules;
using Gadgeteer.Interfaces;

namespace Robot_F6
{
    public class L298_H_BridgeDriver : IDualMotorDriver
    {
        public enum Mode { Slow, Normal, Fast }
        public enum MotionDirection { Stop, LeftForwards, Forwards, RightForwards, LeftBackwards, Backwards, RightBackwards, TurnClockwise, TurnCounterClockwise, Break }
        private Mode _mode = Mode.Normal;
        private MotionDirection _motionDirection = MotionDirection.Stop;
        private IMotor _motor1;
        private IMotor _motor2;

        private uint nThStepHighFast = 1;
        private uint nThStepHighSlow = 4;

        public L298_H_BridgeDriver(int socketNumber, DigitalOutput m1a, DigitalOutput m1b, DigitalOutput m2a, DigitalOutput m2b)
        {
            _motor1 = F6Factory.CreateMotor(m1a, m1b);
            _motor2 = F6Factory.CreateMotor(m2a, m2b);
        }


        public void SetMotionDirection(MotionDirection motionDirection)
        {
            switch (motionDirection)
            {
                case MotionDirection.Backwards:
                    MoveBackwards();
                    break;
                case MotionDirection.Break:
                    Break();
                    break;
                case MotionDirection.Forwards:
                    MoveForwards();
                    break;
                case MotionDirection.LeftBackwards:
                    MoveLeftBackwards();
                    break;
                case MotionDirection.LeftForwards:
                    MoveLeftForwards();
                    break;
                case MotionDirection.RightBackwards:
                    MoveRightBackwards();
                    break;
                case MotionDirection.RightForwards:
                    MoveRightForwards();
                    break;
                case MotionDirection.Stop:
                    Stop();
                    break;
                case MotionDirection.TurnClockwise:
                    TurnClockwise();
                    break;
                case MotionDirection.TurnCounterClockwise:
                    TurnCounterClockwise();
                    break;
            }
        }


        /// <summary>
        /// Set M1 and M2 to move forwards
        /// </summary>
        public void MoveForwards()
        {
            _motor1.SetMotorState(MotorState.MoveForwards);
            _motor1.NthStepActive = nThStepHighFast;
            _motor2.SetMotorState(MotorState.MoveForwards);
            _motor2.NthStepActive = nThStepHighFast;

            _motionDirection = MotionDirection.Forwards;
        }

        /// <summary>
        /// Set M1 and M2 to move left forwards
        /// </summary>
        public void MoveLeftForwards()
        {
            _motor1.SetMotorState(MotorState.MoveForwards);
            _motor1.NthStepActive = nThStepHighSlow;
            _motor2.SetMotorState(MotorState.MoveForwards);
            _motor2.NthStepActive = nThStepHighFast;

            _motionDirection = MotionDirection.LeftForwards;
        }

        /// <summary>
        /// Set M1 and M2 to move right forwards
        /// </summary>
        public void MoveRightForwards()
        {
            _motor1.SetMotorState(MotorState.MoveForwards);
            _motor1.NthStepActive = nThStepHighFast;
            _motor2.SetMotorState(MotorState.MoveForwards);
            _motor2.NthStepActive = nThStepHighSlow;

            _motionDirection = MotionDirection.RightForwards;
        }


        /// <summary>
        /// Set M1 and M2 to move backwards
        /// </summary>
        public void MoveBackwards()
        {
            _motor1.SetMotorState(MotorState.MoveBackwards);
            _motor1.NthStepActive = nThStepHighFast;
            _motor2.SetMotorState(MotorState.MoveBackwards);
            _motor2.NthStepActive = nThStepHighFast;

            _motionDirection = MotionDirection.Backwards;
        }

        /// <summary>
        /// Set M1 and M2 to move left backwards
        /// </summary>
        public void MoveLeftBackwards()
        {
            _motor1.SetMotorState(MotorState.MoveBackwards);
            _motor1.NthStepActive = nThStepHighSlow;
            _motor2.SetMotorState(MotorState.MoveBackwards);
            _motor2.NthStepActive = nThStepHighFast;

            _motionDirection = MotionDirection.LeftBackwards;
        }

        /// <summary>
        /// Set M1 and M2 to move right backwards
        /// </summary>
        public void MoveRightBackwards()
        {
            _motor1.SetMotorState(MotorState.MoveBackwards);
            _motor1.NthStepActive = nThStepHighFast;
            _motor2.SetMotorState(MotorState.MoveBackwards);
            _motor2.NthStepActive = nThStepHighSlow;

            _motionDirection = MotionDirection.RightBackwards;
        }

        /// <summary>
        /// Set M1 and M2 to turn clockwise
        /// </summary>
        public void TurnClockwise()
        {
            _motor1.SetMotorState(MotorState.MoveForwards);
            _motor1.NthStepActive = nThStepHighFast;
            _motor2.SetMotorState(MotorState.MoveBackwards);
            _motor2.NthStepActive = nThStepHighFast;

            _motionDirection = MotionDirection.TurnClockwise;
        }

        /// <summary>
        /// Set M1 and M2 to turn counter clockwise
        /// </summary>
        public void TurnCounterClockwise()
        {
            _motor1.SetMotorState(MotorState.MoveBackwards);
            _motor1.NthStepActive = nThStepHighFast;
            _motor2.SetMotorState(MotorState.MoveForwards);
            _motor2.NthStepActive = nThStepHighFast;

            _motionDirection = MotionDirection.TurnCounterClockwise;
        }

        /// <summary>
        /// Set M1 and M2 to stop
        /// </summary>
        public void Stop()
        {
            _motor1.Stop();
            _motor2.Stop();

            _motionDirection = MotionDirection.Stop;
        }

        /// <summary>
        /// Set M1 and M2 to break
        /// </summary>
        public void Break()
        {
            _motor1.Break();
            _motor2.Break();

            _motionDirection = MotionDirection.Break;
        }

        /// <summary>
        /// Move M1 and M2 one step
        /// </summary>
        public void Step()
        {
            _motor1.Step();
            _motor2.Step();
        }

        public void Fast()
        {
            nThStepHighFast = 1;
            nThStepHighSlow = 2;
            SetMotionDirection(_motionDirection);
        }

        public void Slow()
        {
            nThStepHighFast = 4;
            nThStepHighSlow = 8;
            SetMotionDirection(_motionDirection);
        }
        public void NormalSpeed()
        {
            nThStepHighFast = 2;
            nThStepHighSlow = 4;
            SetMotionDirection(_motionDirection);
        }
    }
}
