﻿using System;
using System.Collections;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Controls;
using Microsoft.SPOT.Presentation.Media;
using Microsoft.SPOT.Touch;

using Gadgeteer.Networking;
using GT = Gadgeteer;
using GTM = Gadgeteer.Modules;
using Gadgeteer.Interfaces;
using Gadgeteer.Modules.GHIElectronics;
using Gadgeteer;
using Gadgeteer.Modules.Seeed;

namespace Robot_F6
{
    public partial class Program
    {
        private IDualMotorDriver _iDualMotorDriver;
        private Distance_US3 _distanceMeterFront;
        private TemperatureHumidityModuleWrapper _temperatureHumidityModuleWrapper;
        private IWebRequestHandler _webRequestHandler;
        private int _intervalmsec = 100;
        private int _counter = 0;
        const int mininumDistance = 25;
        public double Temperature { get; set; }
        public double Humidity { get; set; }
        private bool autoPilot = false;

        void ProgramStarted()
        {
            wifi_RN171.Reboot();

            int socketNumber = 4;

            // Distance_US3 uses pin3 and pin4. Other IO pins are unused.
            _distanceMeterFront = new Distance_US3(socketNumber);
            
            DigitalOutput m1a = extender.SetupDigitalOutput(Socket.Pin.Nine,  false);
            DigitalOutput m1b = extender.SetupDigitalOutput(Socket.Pin.Eight, false);
            DigitalOutput m2a = extender.SetupDigitalOutput(Socket.Pin.Seven, false);
            DigitalOutput m2b = extender.SetupDigitalOutput(Socket.Pin.Six, false);
            _iDualMotorDriver = F6Factory.CreateDualMotorDriver(socketNumber, m1a, m1b, m2a, m2b);

            _temperatureHumidityModuleWrapper = new TemperatureHumidityModuleWrapper(temperatureHumidity);
            
            _webRequestHandler = F6Factory.CreateWebRequestHandler();

            GT.Timer timer = new GT.Timer(1000, GT.Timer.BehaviorType.RunOnce);
            timer.Tick += timer_FirstTick;
            timer.Start();

            Debug.Print("Robot F6 started...");
        }

        public bool IsRunning { get; set; }

        private void timer_FirstTick(GT.Timer timer)
        {
            timer.Stop();
            timer.Tick -= timer_FirstTick;

            SetupWebRequestHandler();
            SetupWifi();
            SetupTemperatureAndHumidityMeasurements();
            HelloWorld();

            StartIntervalTimer();
        }

        private void StartIntervalTimer()
        {
            GT.Timer intervalTimer = new GT.Timer(_intervalmsec, GT.Timer.BehaviorType.RunContinuously);
            intervalTimer.Tick += intervalTimer_Tick;
            intervalTimer.Start();
        }

        private void HelloWorld()
        {
            _iDualMotorDriver.TurnClockwise();
            Thread.Sleep(1000);
            _iDualMotorDriver.Stop();
            Thread.Sleep(1000);
            _iDualMotorDriver.TurnCounterClockwise();
            Thread.Sleep(1000);
            _iDualMotorDriver.Stop();
        }

        private void SetupTemperatureAndHumidityMeasurements()
        {
            temperatureHumidity.MeasurementComplete += temperatureHumidity_MeasurementComplete;
            temperatureHumidity.StartContinuousMeasurements();
        }

        private void SetupWifi()
        {
            wifi_RN171.SetDebugLevel(GTM.GHIElectronics.WiFi_RN171.DebugLevel.DebugAll);
            wifi_RN171.Initialize(GTM.GHIElectronics.WiFi_RN171.SocketProtocol.TCP_Server);
            wifi_RN171.EnableHttpServer(); //Enable HTTP Parsing
            wifi_RN171.HttpRequestReceived += new GTM.GHIElectronics.WiFi_RN171.HttpRequestReceivedHandler(wifi_RN171_HttpRequestReceived);
        }

        private void SetupWebRequestHandler()
        {
            _webRequestHandler.NorthCommand = new MoveForwardsCommand(_iDualMotorDriver);
            _webRequestHandler.NorthEastCommand = new MoveRightForwardsCommand(_iDualMotorDriver);
            _webRequestHandler.EastCommand = new TurnClockwiseCommand(_iDualMotorDriver);
            _webRequestHandler.SouthEastCommand = new MoveRightBackwardsCommand(_iDualMotorDriver);
            _webRequestHandler.SouthCommand = new MoveBackwardsCommand(_iDualMotorDriver);
            _webRequestHandler.SouthWestCommand = new MoveLeftBackwardsCommand(_iDualMotorDriver);
            _webRequestHandler.WestCommand = new TurnCounterClockwiseCommand(_iDualMotorDriver);
            _webRequestHandler.NorthWestCommand = new MoveLeftForwardsCommand(_iDualMotorDriver);
            _webRequestHandler.FireCommand = new DelegatedCommand(new DelegatedCommand.DoSomething(this.ToggleAutoPilot));
            _webRequestHandler.SlowCommand = new SlowCommand(_iDualMotorDriver);
            _webRequestHandler.FastCommand = new FastCommand(_iDualMotorDriver);
            _webRequestHandler.StopCommand = new StopCommand(_iDualMotorDriver);
            _webRequestHandler.BreakCommand = new BreakCommand(_iDualMotorDriver);
            _webRequestHandler.StartCommand = new StartCommand(_iDualMotorDriver);

            _webRequestHandler.TemperatureMeasurer = _temperatureHumidityModuleWrapper;
            _webRequestHandler.HumidityMeasurer = _temperatureHumidityModuleWrapper;
        }

        

        private void intervalTimer_Tick(GT.Timer intervalTimer)
        {
            intervalTimer.Stop();
            int distance = _distanceMeterFront.GetDistanceInCentimeters();
            if (autoPilot)
            {
                _counter++;
                if (_counter * _intervalmsec < 5000)
                {
                    if (distance < 0 || distance > mininumDistance)
                    {
                        _iDualMotorDriver.MoveForwards();
                    }
                    else
                    {
                        _iDualMotorDriver.TurnClockwise();
                    }
                }
                else if (_counter * _intervalmsec < 7500)
                {
                    _iDualMotorDriver.Stop();
                }
                else
                {
                    _counter = 0;
                }
            }
            //else if (distance > 0 && distance < mininumDistance)
            //{
            //    autoPilot = true;
            //}
            else
            {
                _iDualMotorDriver.Step();
            }
            intervalTimer.Start();
        }

        private void temperatureHumidity_MeasurementComplete(GTM.Seeed.TemperatureHumidity sender, double temperature, double relativeHumidity)
        {
            _temperatureHumidityModuleWrapper.SetHumidity(relativeHumidity);
            _temperatureHumidityModuleWrapper.SetTemperature(temperature);
        }

        public void wifi_RN171_HttpRequestReceived(GTM.GHIElectronics.HttpStream request)
        {
            _webRequestHandler.HandleRequest(request);
        }

        private void ToggleAutoPilot()
        {
            this.autoPilot = !autoPilot;
        }
    }
}
